package edu.odu.cs.zeil.AlgAE.Snapshot;

import java.awt.Color;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

public class Entity implements Serializable {
	/**
	 * The objects immediately contained within this one
	 */
	protected List<EntityIdentifier> components;
	
	/**
	 * The objects that this one points to
	 */
	protected List<Connector> connections;

	

	protected EntityIdentifier id;
	protected String label;
	protected String value;
	protected Color color;
	protected int maxComponentsPerRow;
	

	/**
	 * Create an entity representing a "standalone" object
	 * that is not a component of some larger entity
	 * @param id    object to be represented by this new entity
	 */
	public Entity (Identifier id) {
		components = new LinkedList<EntityIdentifier>();
		connections = new LinkedList<Connector>();
		this.id = new EntityIdentifier(id);
		label = "";
		value = "";
		color = Color.lightGray;
		maxComponentsPerRow = 1;
	}
	
	/**
	 * Create an entity representing a named "standalone" object
	 * that is not a component of some larger entity
	 * @param id    object to be represented by this new entity
	 * @param label a descriptive name for this entity
	 */
	public Entity (Identifier id, String label) {
		components = new LinkedList<EntityIdentifier>();
		connections = new LinkedList<Connector>();
		this.id = new EntityIdentifier(id,label);
		this.label = label;
		value = "";
		color = Color.lightGray;
		maxComponentsPerRow = 1;
	}

	
	/**
	 * Create an entity representing an object that is a component
	 * of a larger entity.
	 * 
	 * @param id   object to be represented by this new entity
	 * @param container  entity that is considered to contain this one as a component
	 * @param componentLabel  a string differentiating this component from all others of the same container
	 */
	public Entity (Identifier id, Entity container, String componentLabel) {
		components = new LinkedList<EntityIdentifier>();
		connections = new LinkedList<Connector>();
		this.id = new EntityIdentifier(id, container.getEntityIdentifier(), componentLabel);
		label = componentLabel;
		value = "";
		color = Color.lightGray;
		maxComponentsPerRow = 1;
	}
	
	
	/**
	 * Create an entity representing an object that is a component
	 * of a larger entity.
	 * 
	 * @param id   object to be represented by this new entity
	 * @param container  entity that is considered to contain this one as a component
	 * @param componentLabel  a string differentiating this component from all others of the same container
	 */
	public Entity (Identifier id, EntityIdentifier container, String componentLabel) {
		components = new LinkedList<EntityIdentifier>();
		connections = new LinkedList<Connector>();
		this.id = new EntityIdentifier(id, container, componentLabel);
		label = componentLabel;
		value = "";
		color = Color.lightGray;
		maxComponentsPerRow = 1;
	}
	

	public List<EntityIdentifier> getComponents() {
		return components;
	}
	
	public List<Connector> getConnections() {
		return connections;
	}

	
	public String getLabel() {
		return label;
	}

	public Entity setValue(String value) {
		this.value = value;
		return this;
	}

	public String getValue() {
		return value;
	}
	
	public String getDescription()
	{
		StringBuffer result = new StringBuffer();
		if (label != null && label.length() > 0 && label.charAt(0) >= ' ') {
			result.append (label);
			result.append (": ");
		}
		if (value != null && value.length() > 0) {
			result.append(value);
		}
		return result.toString();
	}

	public Identifier getObjectIdentifier() {
		return id.getObjectIdentifier();
	}

	public EntityIdentifier getContainer() {
		return id.getContainer();
	}

	
	public EntityIdentifier getEntityIdentifier()
	{
		return id;
	}
	
	public Entity setColor(Color color) {
		this.color = color;
		return this;
	}

	public Color getColor() {
		return color;
	}

	public Entity setMaxComponentsPerRow(int maxComponentsPerRow) {
		this.maxComponentsPerRow = maxComponentsPerRow;
		return this;
	}

	public int getMaxComponentsPerRow() {
		return maxComponentsPerRow;
	}
	

	
	public String toString() {
		StringBuffer result = new StringBuffer();
		
		result.append(id);
		result.append(": ");
		
		String description = value;
		if (label != null && label.length() > 0) {
			description = label + ": " + value;
		}
		result.append (description);
		
		result.append ("{eid:" + id + ", label:" + label + ", value:" + value + ", #comp:" + components.size() + ", #conn:" + connections.size() + "}");
		return result.toString();
	}
	
	
	

}
