//  This is a simplified and slimmed-down version of
//  the proposed ANSI standard C++ string class, intended for interim use
//  until our compilers catch up with the standard (and our University catches
//  up with those compiler versions!).
//
//  In essence, it is a "by hand" instantiation of the std basic_string
//  template, with all references to the Allocator and Traits classes removed.
//
//  copyright 1995, S J Zeil, Old Dominion University
//
//  Permission is granted to freely copy, modify, and distribute this file
//  as long as this copyright notice is retained in full.
//

#include <algae/config.h>
#include <algae/stdstrng.h>
//#include <memory.h>
#include <string.h>

#ifndef __GNUG__
//#include <mem.h>
#endif



#ifdef _MEMDEBUG
#include <alloc.h>

static void memoryFailure (int line, const void* address)
{
  cout << "Probable use of deleted block at " << (void*)address
       << "in line " << line << endl;
}
static void heapFailure (int line, int code)
{
  cout << "Probable heap corruption detected, code " << code
       << ", in line " << line << endl;
}

string::size_type* string::freed[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

static void heapMonitor(int line)
{
  int code;
  if ((code = string::allStringsOK()) < 0)
    heapFailure (line, -(code+1));
  else if (!string::checkFreeLists())
    heapFailure (line, -9999);
}


string* string::allStrings[string::maxID] = {0};
int string::nextID = 0;

const string::size_type string::barrierValue = 10 + 16 * (10 + 16 * (5 + 16*5));

bool string::OK() const
{
  if ((chars == 0) || (chars[len] != 0))
    return false;
  if (refcnt != emptyStringArea)
    if (*(refcnt-1) != barrierValue)
      return false;
  return true;
}

int string::allStringsOK()
{
  for (int id = 0; id < maxID; id++)
    if ((allStrings[id] != 0) && (!allStrings[id]->OK()))
      return -id - 1;
  return 0;
}


#define MEMCHECK if ((*refcnt == 0) || (*refcnt > 32000)) memoryFailure(__LINE__, this);
#define HEAPCHECK heapMonitor(__LINE__);
#define allocLength(nChars) 3 + nChars / sizeof(size_type)
#define trueCapacity(nunits) (nunits - 2) * sizeof(size_type) - 1
#else
#define MEMCHECK
#define HEAPCHECK
#define allocLength(nChars) 2 + nChars / sizeof(size_type)
#define trueCapacity(nunits) (nunits - 1) * sizeof(size_type) - 1
#endif


inline static string::size_type min (string::size_type x, string::size_type y)
{
  return (x<y)?x:y;
}


const string::size_type string::npos = -1;
// size_type should be an unsigned type, so this will be implicitly converted
//   to the largest value of that type.



string::size_type string::emptyStringArea[2] = {10, 0};
//  Used for "default" empty string value to be shared among many strings.
//  The first element contains a positive reference count (even when no actual
//    strings reference it) so that no attempt is ever made to delete this
//    storage.
//  The second element is all zeros to denote the empty string.

//char* const string::emptystr = (char*)(emptyStringArea+1);
char* const string::emptystr = "";



void string::allocate (size_type numChars,
   size_type* &ptr, size_type &numAllocated)
{
#ifndef _MEMDEBUG
  size_type nunits = allocLength(numChars);
  numAllocated = trueCapacity(nunits);
  ptr = new size_type [nunits];
#else
  int i = 0;
  numAllocated = sizeof(size_type*);
  while (numAllocated < numChars)
    {
     numAllocated *= 2;
     i++;
    }
  size_type nunits = allocLength(numAllocated);
  numAllocated = trueCapacity(nunits);
  if (freed[i] == 0)
     ptr = new size_type [nunits];
  else
     {
      ptr = freed[i];
      freed[i] = *((size_type**)(ptr+1));
     }

#endif
}

void string::release (string::size_type* ptr, string::size_type numChars)
{
#ifndef _MEMDEBUG
  delete [] ptr;
#else
  int i = 0;
  size_type numAllocated = sizeof(size_type*);
  while (numAllocated < numChars)
    {
     numAllocated *= 2;
     i++;
    }
   int limit = allocLength(numAllocated);
   for (int j = 0; j < limit; j++)
     ptr[j] = barrierValue;
   *((size_type**)(ptr+1)) = freed[i];
   freed[i] = ptr;
#endif
}


#ifdef _MEMDEBUG
bool string::checkFreeLists ()
{
  size_type sz = sizeof(size_type*);
  for (int i = 0; i < 16; i++)
    {
     size_type* ptr = freed[i];
     while (ptr != 0)
       {
        if (*ptr != barrierValue)
          return false;
        int j = 1 + sizeof(size_type*) / sizeof(size_type);
        for (int k = j; k < allocLength(sz); k++)
          if (ptr[k] != barrierValue)
            return false;
        ptr = *((size_type**)(ptr+1));
       }
     sz *= 2;
    }
  return true;
}
#endif


void string::rep_alloc (size_type requested)
{
  allocate (requested, refcnt, capacty);
#ifdef _MEMDEBUG
  *(refcnt) = barrierValue;
  refcnt += 1;
#endif
  size_type *p = refcnt + 1;
  chars = (char*)p;
  *refcnt = 1;
  chars[len] = 0;
HEAPCHECK
}




void string::rep_free (size_type* &refcnt, size_type capac)
{
#ifndef _MEMDEBUG
  release (refcnt, capac-1);
#else
  release (refcnt-1, capac-1);
#endif
HEAPCHECK
}



void string::
rep_append (const char* from, size_type n)
{        MEMCHECK
  memcpy (chars+len, from, n);
  len += n;
  chars[len] = 0;
HEAPCHECK
}


void string::
rep_insert(const char* from, size_type n, size_type at)
{        MEMCHECK
  size_type newlen = len + n;
  size_type j = newlen;
  for (size_type i = len+1; i > at;)
    {
      i--;
      chars[j] = chars[i];
      j--;
    }
  memcpy (chars+at, from, n);
  len = newlen;
HEAPCHECK
}


void string::
rep_erase(size_type at, size_type n)
{        MEMCHECK
  size_type i = at;
  for (size_type j = at+n; j <= len; j++)
    {
      chars[i] = chars[j];
      i++;
    }
  len -= n;
HEAPCHECK
}


string::size_type string::
rep_find (size_type start,
      const char* s, size_type n) const
{        MEMCHECK
  if (len >= n)
    {
      size_type limit = len - n;
      for (size_type pos = start; pos <= limit; pos++)
        {
         if (memcmp(chars+pos, s, n) == 0)
            return pos;
       }
    }
  return npos;
}


string::size_type string::
rep_rfind (size_type finish,
      const char* s, size_type n) const
{        MEMCHECK
  size_type start;
  if (finish < npos)
    start = min(len, finish+1);
  else
    start = len;

  if (start >= n)
    {
      for (size_type pos = start-n+1; pos > 0;)
        {
          pos--;
          if (memcmp(chars+pos, s, n) == 0)
            return pos;
        }
    }
  return npos;
}



string::size_type string::
rep_scan (size_type start, const bool* s) const
{        MEMCHECK
  for (size_type pos = start; pos < len; pos++)
    {
      if (s[(unsigned)chars[pos]])
        return pos;
    }
  return npos;
}

string::size_type string::
rep_rscan (size_type finish, const bool* s) const
{        MEMCHECK
  size_type start;
  if (finish < npos)
    start = min(len, finish+1);
  else
    start = len;
  for (size_type pos = start; pos > 0;)
    {
      pos--;
      if (s[(unsigned)chars[pos]])
        return pos;
    }
  return npos;
}




void string::removeReference(size_type* &rep, size_type capac)
{        MEMCHECK
 --(*rep);
 if (*rep == 0)
    rep_free(rep, capac);
HEAPCHECK
}


void string::
getUniqueBlock (size_type cap, bool preserveValue)
{
  if (preserveValue && (cap < len))
    cap = len;
  if ((*refcnt > 1) || (capacty < cap) || (cap < capacty/16))
    {
      size_type request = capacty;
      if ((capacty == 0) || (cap < capacty/16))
        request = 1;
      while (request < cap)
        request *= 2;
      char *ch = chars;
      if (preserveValue)
        {
         rep_alloc (request);
         memcpy (chars, ch, len+1);
        }
      else
        {
         len = 0;
         rep_alloc (request);
        }
      chars[len] = 0;
    }
HEAPCHECK
}


char* string::
nonConstOp()
{        MEMCHECK
  size_type* oldRef = refcnt;
  size_type oldcap = capacty;
  getUniqueBlock (len, true);
  if (oldRef != refcnt)
    {
      removeReference(oldRef, oldcap);
    }
HEAPCHECK
  return chars;
}


void string::markCharSet (bool* set, const char* s, size_type n, bool value)
{
  for (unsigned c = 0; c <= 255; c++)
    {
      set[c] = !value;
    }
  for (size_type i = 0; i < n; i++)
    {
      set[(unsigned)s[i]] = value;
    }
}






string::string():
  len(0), capacty(0), refcnt(emptyStringArea),
  chars(emptystr)
{
  ++(*refcnt);
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}


string::string (const string& str):
     len(str.len), capacty(str.capacty), chars(str.chars), refcnt(str.refcnt)
{
  ++(*refcnt);
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}


string::string(const string& str, size_type pos,
               size_type n)
{
  assert (pos <= str.len);
  if ((pos == 0) && (n >= str.len))
    {
      len = str.len;
      capacty = str.capacty;
      refcnt = str.refcnt;
      chars = str.chars;
      ++(*refcnt);
    }
  else
    {
      size_type nToCopy = min(n, str.len-pos);
      len = 0;
      rep_alloc (nToCopy);
      rep_append(str.chars+pos, nToCopy);
    }
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}

string::string(const char* s, size_type n)
{
   assert ((s != 0) && (n < npos));
   size_type nToCopy = min(n, strlen(s));
   len = 0;
   rep_alloc (nToCopy);
   rep_append(s, nToCopy);
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}


string::string(const char* s)
{
   assert (s != 0);
   size_type nToCopy = strlen(s);
   len = 0;
   rep_alloc (nToCopy);
   rep_append(s, nToCopy);
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}

string::string(size_type n, char c)
{
  len = n;
  rep_alloc (n);
  memset (chars, c, n);
  chars[n] = 0;
#ifdef _MEMDEBUG
  id = nextID;
  nextID = (nextID + 1) % maxID;
  allStrings[id] = this;
#endif
HEAPCHECK
}



string::~string()
{        MEMCHECK
#ifdef _MEMDEBUG
  allStrings[id] = 0;
#endif
  removeReference(refcnt, capacty);
}



void string::
uncheckedAssign(const char* s, size_type n)
{        MEMCHECK
  size_type* oldref = refcnt;
  size_type oldcap = capacty;
  getUniqueBlock (n, false);
  len = 0;
  rep_append (s, n);
  if (oldref != refcnt)
    {
      removeReference(oldref, oldcap);
    }
HEAPCHECK
}


string& string::
assign(const string& str, size_type pos, size_type n)
{        MEMCHECK
  assert (pos <= str.len);
  if (this != &str)
    {
      if ((pos == 0) && (n >= str.len))
        {
          size_type* oldref = refcnt;
          size_type oldcap = capacty;
          len = str.len;
          capacty = str.capacty;
          chars = str.chars;
          refcnt = str.refcnt;
          ++(*refcnt);
          removeReference(oldref, oldcap);
        }
      else
        {
          size_type nToCopy = min (n, str.len-pos);
          uncheckedAssign (str.chars+pos, nToCopy);
        }
    }
  else if ((pos > 0) || (n < len))
    {
      assign (substr(pos, n));
    }
  
HEAPCHECK
  return *this;
}


string& string::
assign(const char* s, size_type n)
{        MEMCHECK
  size_type nToCopy = min (n, strlen(s));
  uncheckedAssign (s, nToCopy);
HEAPCHECK
  return *this;
}

  
string& string::
assign(size_type n, char c)
{        MEMCHECK
  size_type* oldref = refcnt;
  size_type oldcap = capacty;
  getUniqueBlock (n, false);
  len = n;
  memset (chars, c, n);
  chars[n] = 0;
  if (oldref != refcnt)
    {
      removeReference(oldref, oldcap);
    }
HEAPCHECK
  return *this;
}
  

string& string::
append(const string& str, size_type pos, size_type n)
{        MEMCHECK
  assert (pos <= str.len);
  size_type nToCopy = min (n, str.len-pos);
  if (nToCopy > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len+nToCopy, true);
      rep_append (str.chars+pos, nToCopy);
      if (oldref != refcnt)
        {
          removeReference(oldref, oldcap);
        }
    }
HEAPCHECK
  return *this;
}  

string& string::
append(const char* s, size_type n)
{        MEMCHECK
  size_type nToCopy = min (n, strlen(s));
  if (nToCopy > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len+nToCopy, true);
      rep_append (s, nToCopy);
      if (oldref != refcnt)
        {
          removeReference(oldref, oldcap);
        }
    }
HEAPCHECK
  return *this;
}  
  
  
string& string::
append(size_type n, char c)
{        MEMCHECK
  if (n > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len+n, true);
      memset (chars+len, c, n);
      len += n;
      chars[len] = 0;
      if (oldref != refcnt)
        {
          removeReference(oldref, oldcap);
        }
    }
HEAPCHECK
  return *this;
}  



string& string::
insert(size_type pos1, const string& str,
       size_type pos2, size_type n)
{        MEMCHECK
  assert (pos1 <= len);
  assert (pos2 <= str.len);
  if (n > 0)
    {
      if (&str == this)
        {
          string temp = str.substr(pos2, n);
          return insert (pos1, temp, 0, npos);
        }
      else
        {
          size_type nToCopy = min(n, str.len-pos2);
          size_type* oldref = refcnt;
          size_type oldcap = capacty;
          getUniqueBlock (len+nToCopy, true);
          rep_insert (str.chars+pos2, nToCopy, pos1);
          if (oldref != refcnt)
            {
             removeReference(oldref, oldcap);
           }
        }
    }
HEAPCHECK
  return *this;
}

      
      
string& string::
insert(size_type pos, const char* s, size_type n)
{        MEMCHECK
  assert (pos <= len);
  size_type nToCopy = min(n, strlen(s));
  if (nToCopy > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len+nToCopy, true);
      rep_insert (s, nToCopy, pos);
      if (oldref != refcnt)
        {
         removeReference(oldref, oldcap);
       }
    }
HEAPCHECK
  return *this;
}
  

string& string::
insert(size_type pos, const char* s)
{        MEMCHECK
  return insert(pos, s, npos);
}


string& string::
insert(size_type pos, size_type n, char c)
{        MEMCHECK
  assert (pos <= len);
  if (n > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len+n, true);
      rep_insert (chars, n, pos);
      memset (chars+pos, c, n);
      if (oldref != refcnt)
        {
         removeReference(oldref, oldcap);
       }
    }
HEAPCHECK
  return *this;
}
  

string& string::
erase(size_type pos, size_type n)
{        MEMCHECK
  assert (pos <= len);
  size_type nToRemove = min(n, len-pos);
  if (nToRemove > 0)
    {
      size_type* oldref = refcnt;
      size_type oldcap = capacty;
      getUniqueBlock (len-nToRemove, true);
      rep_erase (pos, nToRemove);
      if (oldref != refcnt)
        {
         removeReference(oldref, oldcap);
        }
    }
HEAPCHECK
  return *this;
}


string& string::
erase(iterator first, iterator last)
{        MEMCHECK
  size_type pos = first - begin();
  size_type n = last - first;
  return erase(pos, n);
}


string& string::
replace(size_type pos1, size_type n1,
        const string& str, size_type pos2,
        size_type n2)
{        MEMCHECK
  assert (pos1 <= len);
  assert (pos2 <= str.len);
  if (this == &str)
    {
      string s = str.substr(pos2, n2);
      erase (pos1, n1);
      insert (pos1, s);
    }
  else
    {
      erase (pos1, n1);
      insert (pos1, str, pos2, n2);
    }
HEAPCHECK
  return *this;
}

string& string::
replace(size_type pos, size_type n1, const char* s,
        size_type n2)
{        MEMCHECK
  erase (pos, n1);
  insert (pos, s, n2);
HEAPCHECK
  return *this;
}

string& string::replace(size_type pos, size_type n1, const char* s)
{        MEMCHECK
  erase (pos, n1);
  insert (pos, s);
HEAPCHECK
  return *this;
}

string& string::replace(size_type pos, size_type n, char c)
{        MEMCHECK
  erase (pos, n);
  insert (pos, 1, c);
HEAPCHECK
  return *this;
}

string& string::replace(iterator first, iterator last, const string& str)
{        MEMCHECK
  size_type pos = first - begin();
  size_type n = last - first;
  assert (n <= len-pos);
  string s = str;
  erase (pos, n);
  insert (pos, s);
HEAPCHECK
  return *this;
}


string& string::replace(iterator first, iterator last,
                        const char* s, size_type n)
{        MEMCHECK
  size_type pos = first - begin();
  size_type n1 = last - first;
  assert (n1+pos <= len);
  erase (pos, n1);
  insert (pos, s, n);
HEAPCHECK
  return *this;
}

string& string::replace(iterator first, iterator last, size_type n, char c)
{        MEMCHECK
  size_type pos = first - begin();
  size_type n1 = last - first;
  assert (n1+pos <= len);
  erase (pos, n1);
  insert (pos, n, c);
HEAPCHECK
  return *this;
}


string::size_type string::max_size() const 
{        MEMCHECK
HEAPCHECK
  return 32767;
}


string::size_type string::copy(char* s, size_type n, size_type pos)
{        MEMCHECK
  assert (pos <= len);
  size_type nToCopy = min(n, len-pos);
  memcpy (s, chars+pos, nToCopy);
  s[nToCopy] = 0;
HEAPCHECK
  return nToCopy;
}


void string::
swap(string& str)
{        MEMCHECK
  {
    size_type temp = len;
    len = str.len;
    str.len = temp;

    temp = capacty;
    capacty = str.capacty;
    str.capacty = temp;
  }
  {
    char* temp = chars;
    chars = str.chars;
    str.chars = temp;
  }
  {
    size_type* temp = refcnt;
    refcnt = str.refcnt;
    str.refcnt = temp;
  }
HEAPCHECK
}


const char* string::
data() const  // explicit
{        MEMCHECK
HEAPCHECK
  return c_str();
}


string::size_type string::
find (const string& str, size_type pos) const
{        MEMCHECK
  assert (pos <= len);
HEAPCHECK
  return rep_find(pos, str.chars, str.len);
}

string::size_type string::
find (const char* s, size_type pos, size_type n) const
{        MEMCHECK
  assert (pos <= len);
  size_type searchWidth = min(n, strlen(s));
HEAPCHECK
  return rep_find(pos, s, searchWidth);
}

string::size_type string::
find (const char* s, size_type pos) const
{        MEMCHECK
  assert (pos <= len);
  size_type searchWidth = strlen(s);
HEAPCHECK
  return rep_find(pos, s, searchWidth);
}
  
string::size_type string::
find (char c, size_type pos) const
{        MEMCHECK
  return find(&c, pos, 1);
}




string::size_type string::
rfind (const string& str, size_type pos) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
HEAPCHECK
  return rep_rfind(searchpos, str.chars, str.len);
}

string::size_type string::
rfind (const char* s, size_type pos, size_type n) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
  size_type searchWidth = min(n, strlen(s));
HEAPCHECK
  return rep_rfind(searchpos, s, searchWidth);
}

string::size_type string::
rfind (const char* s, size_type pos) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
  size_type searchWidth = strlen(s);
HEAPCHECK
  return rep_rfind(searchpos, s, searchWidth);
}
  
string::size_type string::
rfind (char c, size_type pos) const
{        MEMCHECK
  return rfind(&c, pos, 1);
}



string::size_type string::
find_first_of(const string& str, size_type pos) const
{        MEMCHECK
  assert (pos <= len);
  bool set[256];
  markCharSet (set, str.chars, str.len, true);
HEAPCHECK
  return rep_scan(pos, set);
}

string::size_type string::
find_first_of(const char* s, size_type pos, size_type n) const
{        MEMCHECK
  assert (pos <= len);
  bool set[256];
  markCharSet (set, s, min(n,strlen(s)), true);
HEAPCHECK
  return rep_scan(pos, set);
}
  
string::size_type string::
find_first_of(const char* s, size_type pos) const
{        MEMCHECK
  return find_first_of (s, pos, npos);
}

string::size_type string::find_first_of(char c, size_type pos) const
{        MEMCHECK
  return find_first_of (&c, pos, 1);
}


string::size_type string::
find_last_of(const string& str, size_type pos) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
  bool set[256];
  markCharSet (set, str.chars, str.len, true);
HEAPCHECK
  return rep_rscan(searchpos, set);
}

string::size_type string::
find_last_of(const char* s, size_type pos, size_type n) const
{        MEMCHECK
  bool set[256];
  markCharSet (set, s, min(n,strlen(s)), true);
HEAPCHECK
  return rep_rscan(pos, set);
}
  
string::size_type string::
find_last_of(const char* s, size_type pos) const
{        MEMCHECK
  return find_last_of (s, pos, npos);
}

string::size_type string::find_last_of(char c, size_type pos) const
{        MEMCHECK
  return find_last_of (&c, pos, npos);
}


string::size_type string::
find_first_not_of(const string& str, size_type pos) const
{        MEMCHECK
  assert (pos <= len);
  bool set[256];
  markCharSet (set, str.chars, str.len, false);
HEAPCHECK
  return rep_scan(pos, set);
}

string::size_type string::
find_first_not_of(const char* s, size_type pos, size_type n) const
{        MEMCHECK
  assert (pos <= len);
  bool set[256];
  markCharSet (set, s, min(n,strlen(s)), false);
HEAPCHECK
  return rep_scan(pos, set);
}
  
string::size_type string::
find_first_not_of(const char* s, size_type pos) const
{        MEMCHECK
  return find_first_not_of (s, pos, npos);
}

string::size_type string::find_first_not_of(char c, size_type pos) const
{        MEMCHECK
  return find_first_not_of (&c, pos, 1);
}


string::size_type string::
find_last_not_of(const string& str, size_type pos) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
  bool set[256];
  markCharSet (set, str.chars, str.len, false);
HEAPCHECK
  return rep_rscan(searchpos, set);
}

string::size_type string::
find_last_not_of(const char* s, size_type pos, size_type n) const
{        MEMCHECK
  size_type searchpos = min(pos, len);
  bool set[256];
  markCharSet (set, s, min(n,strlen(s)), false);
HEAPCHECK
  return rep_rscan(searchpos, set);
}
  
string::size_type string::
find_last_not_of(const char* s, size_type pos) const
{        MEMCHECK
  return find_last_not_of (s, pos, npos);
}

string::size_type string::find_last_not_of(char c, size_type pos) const
{        MEMCHECK
  return find_last_not_of (&c, pos, 1);
}


int string::
compare(const string& str) const
{        MEMCHECK
  return compare (0, npos, str);
}


int string::
compare(size_type pos1, size_type n1, const string& str) const
{        MEMCHECK
  assert (pos1 <= len);
  if ((chars == str.chars) && (pos1 == 0) && (n1 >= len))
    return 0;
  else
    {
      size_type nToCompare1 = min(n1, len-pos1);
      size_type nToCompare2 = str.len;
      size_type nToCompare = min(nToCompare1, nToCompare2);
      int result = memcmp(chars+pos1, str.chars, nToCompare);
      if (result == 0)
        {
          if (nToCompare1 < nToCompare2)
            return -1;
          else if (nToCompare1 == nToCompare2)
            return 0;
          else
            return 1;
        }
      else
        return result;
    }
}


int string::
compare(size_type pos1, size_type n1, const string& str,
        size_type pos2, size_type n2) const
{        MEMCHECK
  assert (pos1 <= len);
  assert (pos2 <= str.len);
  if ((chars == str.chars) && (pos1 == pos2) && (n1 == n2))
    return 0;
  else
    {
      size_type nToCompare1 = min(n1, len-pos1);
      size_type nToCompare2 = min(n2, str.len - pos2);
      size_type nToCompare = min(nToCompare1, nToCompare2);

      int result = memcmp(chars+pos1, str.chars+pos2, nToCompare);
      if (result == 0)
        {
          if (nToCompare1 < nToCompare2)
            return -1;
          else if (nToCompare1 == nToCompare2)
            return 0;
          else
            return 1;
        }
      else
        return result;
    }
}


int string::
compare(size_type pos1, size_type n1, const char* s,
        size_type n2) const
{        MEMCHECK
  assert ((pos1 <= len) && (s != 0));

  {
    size_type slen = strlen(s);

      size_type nToCompare1 = min(n1, len-pos1);
      size_type nToCompare2 = min(n2, slen);
      size_type nToCompare = min(nToCompare1, nToCompare2);


      int result = memcmp(chars+pos1, s, nToCompare);
      if (result == 0)
        {
          if (nToCompare1 < nToCompare2)
            return -1;
          else if (nToCompare1 == nToCompare2)
            return 0;
          else
            return 1;
        }
      else
        return result;
    }
}



      
int string::compare(const char* s) const
{        MEMCHECK
  assert (s != 0);

  {
    size_type slen = strlen(s);

    const size_type &nToCompare1 = len;
    size_type &nToCompare2 = slen;
    size_type nToCompare = min(nToCompare1, nToCompare2);

      int result = memcmp(chars, s, nToCompare);
      if (result == 0)
        {
          if (nToCompare1 < nToCompare2)
            return -1;
          else if (nToCompare1 == nToCompare2)
            return 0;
          else
            return 1;
        }
      else
        return result;
    }
}



istream& operator>> (istream& is, string& str)
{
  char c = 0;
  str = string();
  while (is)
    {
      c = is.get();
      if (c > ' ')
        {
          str.append(1, c);
          break;
        }
    }
  while (is)
    {
      c = is.get();
      if (c <= ' ')
        break;
      else
        str.append(1, c);
    }
  if (is)
    is.putback(c);
  return is;
}
  
ostream& operator<< (ostream& os, const string& str)
{
  return (os << str.c_str());
}


istream& getline (istream& is, string& str, char delim)
{
  char c = 0;
  str = string();
  while (is)
    {
      c = is.get();
      if (c == delim)
        break;
      else
        str.append(1, c);
    }
  return is;
}


void string::resize(size_type n, char c)
  //pre:
  //   s1 = s2; s2.resize(n, c)
  //post: s2.size() == n
  //      s1.substr(0, min(n, s1.size())) == s2.substr(0, min(n, s1.size()))
  //      for all i in s1.size()..n-1, s2[i] == c
{        MEMCHECK
  size_type* oldref = refcnt;
  size_type oldcap = capacty;
  getUniqueBlock (n, true);
  for (unsigned i = len; i < n; i++)
    chars[i] = c;
  chars[n] = 0;
  len = n;

  if (oldref != refcnt)
    {
     removeReference(oldref, oldcap);
   }
HEAPCHECK
}


void string::reserve(size_type res_arg)
{        MEMCHECK
  size_type* oldref = refcnt;
  size_type oldcap = capacty;
  getUniqueBlock (res_arg, true);
  if (oldref != refcnt)
    {
     removeReference(oldref, oldcap);
   }
HEAPCHECK
}


  
