package edu.odu.cs.zeil.AlgAE.Client.SourceViewer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;


/**
 * SourceFile.java
 *
 * A description of a file of animated source code
 *
 *
 * @author Steven Zeil
 * @version
 */

public class SourceFile
{
	private static final String SpecialSourceMarker = "//!";
	private Class<?> container;
	private String fileName;
	private String contents;
	private ArrayList<Integer> lineOffsets;
	private int selectedLine;



	public SourceFile(Class<?> container, String fileName) {
		this.fileName = fileName;
		this.container = container;
		contents = "";
		lineOffsets = new ArrayList<Integer>();
		lineOffsets.add(0);
		selectedLine = -1;
		load();
	}




	public String getClassName() {
		return container.getName();
	}

	public String getResourceName() {
		return container.getPackage().getName() + "." + fileName;
	}


	public String getContents() {
		return contents;
	}


	public ArrayList<Integer> getLineOffsets() {
		return lineOffsets;
	}



	public void setSelectedLine(int selectedLine) {
		this.selectedLine = selectedLine;
	}



	public int getSelectedLine() {
		return selectedLine;
	}


	private void load()
	{
		String resourceName = "/" + container.getPackage().getName().replace(".", "/") + "/" + fileName;
		
		InputStream resourceIn = container.getResourceAsStream(resourceName);
		//System.err.println ("resourceIn: " + resourceIn);
		if (resourceIn != null) {
			StringBuffer contentBuf = new StringBuffer();
			BufferedReader in = new BufferedReader (new InputStreamReader(resourceIn));
			int offSet = 0;
			int lineCount = 0;
			lineOffsets.clear();
			try {
				while (in.ready()) {
					lineOffsets.add(offSet);
					String line = in.readLine();
					++lineCount;
					if (line.contains(SpecialSourceMarker)) {
						line = line.substring(line.indexOf(SpecialSourceMarker) + SpecialSourceMarker.length());
						if (line.length() == 0) {
							continue;
						}
					}
					line = line.replace("\t", "    ");
					String lineNum = "" + lineCount;
					while (lineNum.length() < 3) {
						lineNum = "0" + lineNum;
					}
					line = lineNum + ": " + line + "\n";
					contentBuf.append(line);
					offSet += line.length();
				}
				contents = contentBuf.toString();
			} catch (IOException ex) {
				ex.printStackTrace();
			} finally {
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		} else {
			contents = "Could not load " + fileName;
		}
	}



} // SourceFile
