package edu.odu.cs.zeil.AlgAE.Snapshot.Layout;

import java.awt.Color;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import edu.odu.cs.zeil.AlgAE.Animation;
import edu.odu.cs.zeil.AlgAE.Client.Anchors;
import edu.odu.cs.zeil.AlgAE.Client.DataViewer.Frames.Frame;
import edu.odu.cs.zeil.AlgAE.Client.SourceViewer.SourceLocation;
import edu.odu.cs.zeil.AlgAE.Coordinates.Dimension2DDouble;
import edu.odu.cs.zeil.AlgAE.Snapshot.ActivationStack;
import edu.odu.cs.zeil.AlgAE.Snapshot.Component;
import edu.odu.cs.zeil.AlgAE.Snapshot.Connection;
import edu.odu.cs.zeil.AlgAE.Snapshot.Entity;
import edu.odu.cs.zeil.AlgAE.Snapshot.EntityIdentifier;
import edu.odu.cs.zeil.AlgAE.Snapshot.Identifier;
import edu.odu.cs.zeil.AlgAE.Snapshot.Snapshot;
import edu.odu.cs.zeil.AlgAE.Snapshot.TestSnapshot;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.CanBeRendered;
import edu.odu.cs.zeil.AlgAE.Snapshot.Rendering.Renderer;

/**
 * @author zeil
 *
 */
public class TestLayout extends TestCase {

    public TestLayout(String testName) {
        super(testName);
    }

    public static void main( String[] args ) throws Exception {
        TestRunner.run( suite() );
    }

    public static Test suite() {
        return new TestSuite(TestLayout.class);
    }
    
    
    private static Animation a;
    public void setUp() {
    	a = new Animation("dummy", false) {
			@Override
			public void buildMenu() {
			}

			@Override
			public String about() {
				return null;
			}
    	};
    	a.registerInstance(Thread.currentThread());
    }

	private static void checkvar (HashMap<EntityIdentifier, Entity> entities, Object obj, int cnt, String label, String value) {
		assertEquals (cnt, countvar(entities, obj));
		Identifier oid = Identifier.getIdentifierFor(obj, a);
		EntityIdentifier eid;
		eid = new EntityIdentifier(oid, label);
		Entity e = entities.get(eid);
		assertNotNull(e);
		if (label != null)
			assertEquals (label, e.getLabel());
		if (value != null)
			assertEquals(value, e.getValue());	
	}

	private static int countvar (HashMap<EntityIdentifier, Entity> entities, Object obj) {
		Identifier id = Identifier.getIdentifierFor(obj, a);
		int count = 0;
		for (EntityIdentifier eid: entities.keySet()) {
			if (id.equals(eid.getObjectIdentifier())) {
				++count;
			}
		}
		return count;
	}
	
	
	private static void checksize (Layout scene, Object obj, String label, double minx, double maxx, double miny, double maxy) {
		EntityIdentifier eid = new EntityIdentifier(Identifier.getIdentifierFor(obj, a), label);
		Dimension2DDouble sz = scene.getSizeOf(eid);
		double w = sz.getWidth();
		double h = sz.getHeight();
		assertTrue (w >= minx);
		assertTrue (w <= maxx);
		assertTrue (h >= miny);
		assertTrue (h <= maxy);
	}

    
	public static void test_emptySnap() {
		ActivationStack callStack = new ActivationStack(a);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		HashMap<EntityIdentifier, Entity> entities =  scene.getEntities();
		assertEquals (2, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
	}

	public static void test_Snap_global() {
		ActivationStack callStack = a.getActivationStack();
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		HashMap<EntityIdentifier, Entity> entities =  scene.getEntities();
		assertEquals (3, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
		checksize (scene, fortyTwo, "foo", 6, 9, 1, 2);
	}

	private static class String2 implements Renderer<String2>, CanBeRendered<String2> {
		
		public String s;
		
		public String2 (String ss) {
			s = ss;
		}

		@Override
		public Renderer<String2> getRenderer() {
			return this;
		}

		@Override
		public String getValue(String2 obj) {
			return s;
		}

		@Override
		public Color getColor(String2 obj) {
			return Color.blue;
		}

		@Override
		public List<Component> getComponents(String2 obj) {
			LinkedList<Component> c = new LinkedList<Component>();
			c.add(new Component(s, "comp"));
			return c;
		}

		@Override
		public List<Connection> getConnections(String2 obj) {
			return new LinkedList<Connection>();
		}

		@Override
		public int getMaxComponentsPerRow(String2 obj) {
			return 1;
		}
		
	}
	
	public static void test_Scene_components() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String twelve = "12";
		String2 dozen = new String2(twelve);
		callStack.globalVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		HashMap<EntityIdentifier, Entity> entities =  scene.getEntities();
		assertEquals (5, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
		checkvar (entities, dozen, 1, "dozen", "12");
		checksize (scene, fortyTwo, "foo", 6, 9, 1, 2);
		checksize (scene, dozen, "dozen", 9, 12, 2, 4);
	}

	public static void test_Scene_connections() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String twelve = "12";
		String2 dozen = new String2(twelve);
		callStack.globalRefVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		HashMap<EntityIdentifier, Entity> entities =  scene.getEntities();
		assertEquals (6, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 1, "foo", "42");
		checkvar (entities, dozen, 1, null, "12");
	}

	public static void test_Scene_alias() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		String2 dozen = new String2(fortyTwo);
		callStack.globalVar("dozen", dozen);
		Snapshot snap = new Snapshot("description", new SourceLocation(TestSnapshot.class, "foo.java", 23), callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		HashMap<EntityIdentifier, Entity> entities =  scene.getEntities();
		assertEquals (5, entities.keySet().size());
		checkvar (entities, callStack, 1, null, null);
		checkvar (entities, fortyTwo, 2, "foo", "42");
		checkvar (entities, dozen, 1, "dozen", "42");
	}

	public static void test_Scene_render1() {
		ActivationStack callStack = new ActivationStack(a);
		String fortyTwo = "42";
		callStack.globalVar("foo", fortyTwo);
		SourceLocation sloc = new SourceLocation(TestSnapshot.class, "foo.java", 23);
		Snapshot snap = new Snapshot("description", sloc, callStack);
		Anchors anchors = new Anchors();
		Layout scene = new Layout(snap, null, anchors);
		Frame f = scene.toPicture();
		assertEquals ("description", f.getMessage());
		assertEquals (sloc, f.getLocation());
		assertTrue (f.isKey());
		boolean found42 = false;
		Identifier id42 = Identifier.getIdentifierFor(fortyTwo, a);
		boolean foundStack = false;
		Identifier idstack = Identifier.getIdentifierFor(callStack, a);
		for (Iterator<String> it = f.shapeKeys(); it.hasNext(); ) {
			String fid = it.next();
			if (fid.startsWith(id42.toString())) {
				found42 = true;
			}
			if (fid.startsWith(idstack.toString())) {
				foundStack = true;
			}
		}
		assertTrue (found42);
		assertTrue (foundStack);
	}

}
